import xarray, netCDF4
from esgcet.handler_base import ESGPubHandlerBase
import os.path
class ESGPubXArrayHandler(ESGPubHandlerBase):

    @staticmethod
    def xarray_load(map_data):
        datafile = map_data[0][1]
        destpath = os.path.dirname(datafile)

        filespec = f"{destpath}/*.nc"
        return xarray.open_mfdataset(filespec)

    def get_attrs_dict(self, scanobj):
        return scanobj.attrs

    def get_scanfile_dict(self, scandata, mapdata):
        ret = {}
        for rec in mapdata:
            fn = rec['file']
            print(f"TEST: {fn}")
            ds = netCDF4.Dataset(fn)
            ret[fn] = {"tracking_id": ds.tracking_id}
        return ret
    
    def get_variables(self, scanobj):
        res = {}
        for x in scanobj.variables:
            res[x] = scanobj.variables[x].attrs
        return res
    
    def get_variable_list(self, variable):
        return [x for x in variable]

    def _get_time_str(self, timeval):
        if type(timeval.item()) is int:
            x = str(timeval)
            idx = x.index('.')
            return x[:idx] + 'Z'
        else:
            return timeval.item().isoformat() + "Z"

    def set_bounds(self, record, scanobj):

        geo_units = []
        # latitude
        if "latitude" in scanobj.coords:
            lat = scanobj.coords["latitude"]
            record["north_degrees"] = lat[-1].values.item()
            record["south_degrees"] = lat[0].values.item()
            geo_units.append(lat.units)
        # longitude
        if "longitude" in scanobj.coords:
            lon = scanobj.coords["longitude"]
            record["east_degrees"] = lon[-1].values.item()
            record["west_degrees"] = lon[0].values.item()
            geo_units.append(lon.units)
        # time
        if "time" in scanobj.coords:
            ti = scanobj.coords["time"]
            record["datetime_start"] = self._get_time_str(ti[0].values)
            record["datetime_end"] = self._get_time_str(ti[-1].values)
        # plev
        if "plev" in scanobj.coords:
            plev = scanobj.coords["plev"]
            record["height_top"] = plev[0].values.item() 
            record["height_bottom"] = plev[-1].values.item() 
            geo_units.append(plev.units)
        if len(geo_units) > 0:
            record["geo_units"] = geo_units
            



from esgcet.pub_client import publisherClient
import sys, json

from esgcet.pid_cite_pub import ESGPubPidCite
from esgcet.search_check import ESGSearchCheck


import esgcet.logger as logger

log = logger.ESGPubLogger()

def check_for_pid_proj(dset_arr):

    for dset in dset_arr:

        parts = dset.split('.')
        if parts[0].lower() in ["cmip6", "input4mips"]:
            return True            

    return False


def run(args):

    hostname = args["index_node"]
    data_node = args["data_node"]
    verbose = args["verbose"]    
    silent = args["silent"]
    auth = args["auth"]
    cert_fn = args["cert"]
    do_delete = args["delete"]

    pub_log = log.return_logger('Unpublish', args["silent"], args["verbose"])
    searchcheck = ESGSearchCheck(hostname, silent, verbose)
    status = 0

    for dset_id in args["dataset_id_lst"]:

        status += single_unpublish(dset_id, args, pub_log, searchcheck)
    return status

def single_unpublish(dset_id, args, pub_log, searchcheck):

    hostname = args["index_node"]
    do_delete = args["delete"]
    data_node = args["data_node"]
    cert_fn = args["cert"]
    auth = args["auth"]

    second_split = []
    if '|' in dset_id:
        first_split = dset_id.split('|')
        second_split = first_split[0].split('.')
        data_node = first_split[1]
    else:
        second_split = dset_id.split('.')
        dset_id_new = '{}|{}'.format(dset_id, data_node)
        dset_id = dset_id_new


    found, notretracted = searchcheck.run_check(dset_id)

    if not found:
        return(-1)

    if (not notretracted) and (not do_delete):
        pub_log.info("Use --delete to permanently erase the retracted record")
        return(0)

    if "pid_creds" in args and check_for_pid_proj([dset_id]):
        version = second_split[-1][1:]
        master_id = '.'.join(second_split[0:-1])
        pid_module = ESGPubPidCite({}, args["pid_creds"], data_node, False, args["silent"], args["verbose"])
        ret = pid_module.pid_unpublish(master_id, version)
        if not ret:
            pub_log.warning("PID Module did not return success")
    # ensure that dataset id is in correct format, use the set data node as a default
        
    pubCli = publisherClient(cert_fn, hostname, auth=auth, verbose=args["verbose"], silent=args["silent"])

    if do_delete:
        pubCli.delete(dset_id)
    else:
        pubCli.retract(dset_id)
    return(0)





#!/usr/bin/env python
    
from setuptools import setup, find_packages
from pathlib import Path
import os
import sys
import configparser as cfg
from shutil import copyfile


import esgcet
VERSION = esgcet.__version__

print("esgcet version =", VERSION)
HOME = str(Path.home())
FULLPATH = HOME + '/.esg'
DEFAULT_ESGINI = '/esg/config/esgcet'

if not os.path.exists(FULLPATH):
    os.makedirs(FULLPATH)

if os.path.exists(FULLPATH + "/esg.yaml"):
    make_config = False
else:
    make_config = True

if make_config:
    setup(
        name = 'esgcet',
        version = VERSION,
        description = 'ESGCET publication package',
        author = 'Sasha Ames',
        author_email = 'ames4@llnl.gov',
        url = 'http://esgf.llnl.gov',
        install_requires = [
            "requests",
             "esgfpid",
            "ESGConfigParser==1.0.0a1",
            "xarray",
            "netcdf4",
            "dask",
            "pyyaml"
        ],
        packages = find_packages(exclude=['ez_setup']),
        include_package_data = True,
        scripts = [
        ],
        zip_safe = False,                   # Migration repository must be a directory
        entry_points={'console_scripts': ['esgpidcitepub=esgcet.esgpidcitepub:main',
                                          'esgmkpubrec=esgcet.esgmkpubrec:main',
                                          'esgindexpub=esgcet.esgindexpub:main',
                                          'esgpublish=esgcet.pub_internal:main',
                                          'esgupdate=esgcet.esgupdate:main',
                                          'esgmapconv=esgcet.esgmapconv:main',
                                          'esgmigrate=esgcet.migratecmd:main',
                                          'esgunpublish=esgcet.esgunpublish:main']},
        data_files=[(FULLPATH, ['esg.yaml'])]

    )
else:
    setup(
        name = 'esgcet',
        version = VERSION,
        description = 'ESGCET publication package',
        author = 'Sasha Ames',
        author_email = 'ames4@llnl.gov',
        url = 'http://esgf.llnl.gov',
        install_requires = [
            "requests",
             "esgfpid",
            "ESGConfigParser==1.0.0a1"
        ],
        packages = find_packages(exclude=['ez_setup']),
        include_package_data = True,
        scripts = [
        ],
        zip_safe = False,                   # Migration repository must be a directory
        entry_points={'console_scripts': ['esgpidcitepub=esgcet.esgpidcitepub:main',
                                          'esgmkpubrec=esgcet.esgmkpubrec:main',
                                          'esgindexpub=esgcet.esgindexpub:main',
                                          'esgpublish=esgcet.pub_internal:main',
                                          'esgupdate=esgcet.esgupdate:main',
                                          'esgmapconv=esgcet.esgmapconv:main'
                                          'esgmigrate=esgcet.migratecmd:main',
                                          'esgunpublish=esgcet.esgunpublish:main']}
        )



from esgcet.mapfile import ESGPubMapConv
from esgcet.mkd_non_nc import ESGPubMKDNonNC
from esgcet.update import ESGPubUpdate
from esgcet.index_pub import ESGPubIndex
import sys
import esgcet.logger as logger

log = logger.ESGPubLogger()


class BasePublisher(object):

    def __init__(self, argdict):
        self.argdict = argdict
        self.fullmap = argdict["fullmap"]
        self.silent = argdict["silent"]
        self.verbose = argdict["verbose"]
        self.cert = argdict["cert"]
        self.index_node = argdict["index_node"]
        self.data_node = argdict["data_node"]
        self.data_roots = argdict["data_roots"]
        self.globus = argdict["globus"]
        self.dtn = argdict["dtn"]
        self.replica = argdict["replica"]
        self.proj = argdict["proj"]
        self.json_file = argdict["json_file"]
        self.auth = argdict["auth"]
        self.proj_config = argdict["user_project_config"]
        self.verify = argdict["verify"]
        self.mountpoints = argdict["mountpoints"]
        self.project = argdict["proj"]
        self.publog = log.return_logger('Generic Non-NetCDF Publisher', self.silent, self.verbose)

    def cleanup(self):
        pass

    def mapfile(self):

        mapconv = ESGPubMapConv(self.fullmap, project=self.project)
        map_json_data = None
        try:
            map_json_data = mapconv.mapfilerun(self.mountpoints)

        except Exception as ex:
            self.publog.exception("Failed to convert mapfile")
            self.cleanup()
            exit(1)
        return map_json_data

    def mk_dataset(self, map_json_data):
        mkd = ESGPubMKDNonNC(self.data_node, self.index_node, self.replica, self.globus, self.data_roots, self.dtn,
                                self.silent, self.verbose)
        mkd.set_project(self.project)
        try:
            out_json_data = mkd.get_records(map_json_data, self.json_file, user_project=self.proj_config)
        except Exception as ex:
            self.publog.exception("Failed to make dataset")
            self.cleanup()
            exit(1)
        return out_json_data

    def update(self, json_data):
        up = ESGPubUpdate(self.index_node, self.cert, silent=self.silent, verbose=self.verbose, verify=self.verify, auth=self.auth)
        try:
            up.run(json_data)
        except Exception as ex:
            self.publog.exception("Failed to update record")
            self.cleanup()
            exit(1)

    def index_pub(self,dataset_records):
        arch_cfg = None
        if self.argdict["enable_archive"]:
            arch_cfg = { "length" : int(self.argdict["archive_path_length"]) , 
                          "archive_path" : self.argdict["archive_path"]}

        ip = ESGPubIndex(self.index_node, self.cert, silent=self.silent, verbose=self.verbose, verify=self.verify, auth=self.auth, arch_cfg=arch_cfg)
        rc = True
        try:
            rc = ip.do_publish(dataset_records)
        except Exception as ex:
            self.publog.exception("Failed to publish to index node")
            self.cleanup()
            exit(1)
        return rc

    def workflow(self):

        # step one: convert mapfile
        self.publog.info("Converting mapfile...")
        map_json_data = self.mapfile()

        # step two: make dataset
        self.publog.info("Making dataset...")
        out_json_data = self.mk_dataset(map_json_data)

        self.publog.info("Updating...")
        self.update(out_json_data)

        self.publog.info("Running index pub...")
        
        rc = self.index_pub(out_json_data)

        self.publog.info("Done.")

        return rc
